﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Tactile.Graphics.Help;
using Tactile.Graphics.Map;
using Tactile.Graphics.Text;
using Tactile.Graphics.Windows;
using Tactile.Windows.UserInterface;
using Tactile.Windows.UserInterface.Map;

namespace Tactile.Windows.Map
{
    class Window_Data : Map_Window_Base
    {
        const int RANKING_BAR_LENGTH = 56;

        readonly static Vector2 TEAM_WINDOW_LOC = new Vector2(8, 32);
        private int Index = 0;
        private bool ChangingPage = false;
        private int ChangingPageTime = 0;
        private Vector2 PageLoc;
        private List<KeyValuePair<int, int>> Groups = new List<KeyValuePair<int, int>>();

        new private Data_Team_Cursor Cursor;
        private UINodeSet<DataTeamUINode> TeamNodes;
        private TextSprite Banner;
        private Sprite BannerBg, UnitBg, FileBg;
        private Sprite Objective_Window, Game_Data_Window;
        private Data_Ranking_Window RankingWindow;
        private Sprite Objective_Label, Defeat_Label, Turn_Label, Funds_Label, Gold_G, Lv_Label, Hp_Label, Hp_Slash, RatingLabel;
        private Play_Time_Counter Counter;
        private Item_Display LeaderWeapon;
        private TextSprite Victory_Text, Loss_Text, FileLabel, RankingLabel,
            TurnsRankLabel, CombatRankLabel, ExpRankLabel;
        private RightAdjustedText Turn, Funds, FileNumber;
        private TextSprite Leader_Name, StyleText, DifficultyText, TotalRankLetter,
            TurnsRankLetter, CombatRankLetter, ExpRankLetter;
        private Stat_Bar TurnsBar, CombatBar, ExpBar;
        private RightAdjustedText Lvl, Hp, Hp_Max, Rating;
        private Character_Sprite Map_Sprite;
        private Miniface Face;
        private Page_Arrow Left_Page_Arrow, Right_Page_Arrow;

        private Button_Description CancelButton;

        #region Accessors
        private Vector2 target_page_loc { get { return new Vector2(page == 0 ? 0 : 128, 0); } }

        private int page
        {
            get { return Global.game_system.DataPage; }
            set { Global.game_system.DataPage = value; }
        }

        protected override bool ready_for_inputs { get { return base.ready_for_inputs && !ChangingPage; } }
        #endregion

        public Window_Data()
        {
            PageLoc = target_page_loc;
            initialize_sprites();
            update_black_screen();
        }

        private void initialize_sprites()
        {
            // Black Screen
            Black_Screen = new Sprite();
            Black_Screen.texture = Global.Content.Load<Texture2D>(@"Graphics/White_Square");
            Black_Screen.dest_rect = new Rectangle(0, 0, Config.WINDOW_WIDTH, Config.WINDOW_HEIGHT);
            Black_Screen.tint = new Color(0, 0, 0, 255);
            // Banner
            Banner = new TextSprite();
            string str = Global.data_chapters[Global.game_state.chapter_id].FullName;
            Banner.loc = new Vector2(Config.WINDOW_WIDTH, 32) / 2 - new Vector2(Font_Data.text_width(str, Config.CHAPTER_FONT) / 2, 8);
            Banner.SetFont(Config.CHAPTER_FONT, Global.Content, "Data");
            Banner.text = str;
            Banner.stereoscopic = Config.DATA_BANNER_DEPTH;

            int alpha = 7 * 16;
            BannerBg = new Sprite();
            BannerBg.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            BannerBg.loc = new Vector2(64, 0);
            BannerBg.src_rect = new Rectangle(0, 176, 192, 32);
            BannerBg.tint = new Color(alpha, alpha, alpha, 256 - alpha);
            BannerBg.stereoscopic = Config.DATA_BANNER_DEPTH;
            // Unit Bg
            UnitBg = new Sprite();
            UnitBg.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            UnitBg.loc = new Vector2(132, 120);
            UnitBg.src_rect = new Rectangle(0, 272, 128, 72);
            UnitBg.tint = new Color(alpha, alpha, alpha, 256 - alpha);
            UnitBg.stereoscopic = Config.DATA_LEADER_DEPTH;
            // File Bg
            FileBg = new Sprite();
            FileBg.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            FileBg.loc = new Vector2(260, 128);
            FileBg.src_rect = new Rectangle(0, 208, 56, 64);
            FileBg.tint = new Color(alpha, alpha, alpha, 256 - alpha);
            FileBg.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Background
            Background = new Menu_Background();
            Background.texture = Global.Content.Load<Texture2D>(@"Graphics/Pictures/Status_Background");
            (Background as Menu_Background).vel = new Vector2(-0.25f, 0);
            (Background as Menu_Background).tile = new Vector2(3, 2);
            Background.stereoscopic = Config.MAPMENU_BG_DEPTH;
            // Cursor
            Cursor = new Data_Team_Cursor();
            Cursor.stereoscopic = Config.DATA_TEAMS_DEPTH;
            // Objective Window
            Objective_Window = new Sprite();
            Objective_Window.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Objective_Window.loc = new Vector2(132, 32);
            Objective_Window.src_rect = new Rectangle(32, 0, 184, 96);
            Objective_Window.stereoscopic = Config.DATA_WINDOW_DEPTH;
            // Game Data Window
            Game_Data_Window = new Sprite();
            Game_Data_Window.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Game_Data_Window.loc = new Vector2(324, 120);
            Game_Data_Window.src_rect = new Rectangle(104, 96, 112, 72);
            Game_Data_Window.stereoscopic = Config.DATA_DATA_DEPTH;
            // Team Windows/Text
            List<DataTeamUINode> nodes = new List<DataTeamUINode>();
            int y = 0;
            for (int i = 1; i <= Constants.Team.NUM_TEAMS; i++)
                if (Global.game_map.teams[i].Count > 0)
                {
                    List<int> groups = new List<int>();
                    Dictionary<int, int> group_sizes = new Dictionary<int, int>();
                    foreach (int id in Global.game_map.teams[i])
                    {
                        if (!groups.Contains(Global.game_map.units[id].group))
                        {
                            groups.Add(Global.game_map.units[id].group);
                            group_sizes.Add(Global.game_map.units[id].group, 0);
                        }
                        group_sizes[Global.game_map.units[id].group]++;
                    }
                    Groups.Add(new KeyValuePair<int, int>(i, groups.Count));

                    groups.Sort();

                    var groupNames = groups.Select(x => Global.game_map.group_name(i, x)).ToList();
                    var groupCounts = groups.Select(x => group_sizes[x]).ToList();

                    var newNode = new DataTeamUINode(i, groupNames, groupCounts);
                    newNode.loc = TEAM_WINDOW_LOC + new Vector2(0, 0 + y);
                    newNode.stereoscopic = Config.DATA_TEAMS_DEPTH;
                    nodes.Add(newNode);

                    y += (group_sizes.Count + 1) * 16;
                }

            TeamNodes = new UINodeSet<DataTeamUINode>(nodes);
            TeamNodes.CursorMoveSound = System_Sounds.Menu_Move1;
            TeamNodes.SoundOnMouseMove = true;
            // Game Data Window
            RankingWindow = new Data_Ranking_Window();
            RankingWindow.loc = new Vector2(320, 32);
            RankingWindow.width = 120;
            RankingWindow.height = 80;
            RankingWindow.stereoscopic = Config.DATA_TEAMS_DEPTH;
            // Counter
            Counter = new Play_Time_Counter();
            Counter.loc = Game_Data_Window.loc + new Vector2(12, 48);
            Counter.stereoscopic = Config.DATA_DATA_DEPTH;
            // Labels
            Objective_Label = new Sprite();
            Objective_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Objective_Label.loc = Objective_Window.loc + new Vector2(16 - 2, 12 - 2);
            Objective_Label.src_rect = new Rectangle(0, 96, 56, 16);
            Objective_Label.stereoscopic = Config.DATA_WINDOW_DEPTH;
            Defeat_Label = new Sprite();
            Defeat_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Defeat_Label.loc = Objective_Window.loc + new Vector2(16 - 2, 44 - 2);
            Defeat_Label.src_rect = new Rectangle(56, 96, 40, 16);
            Defeat_Label.stereoscopic = Config.DATA_WINDOW_DEPTH;

            Turn_Label = new Sprite();
            Turn_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Turn_Label.loc = Game_Data_Window.loc + new Vector2(8, 8);
            Turn_Label.src_rect = new Rectangle(0, 112, 32, 16);
            Turn_Label.stereoscopic = Config.DATA_DATA_DEPTH;
            Funds_Label = new Sprite();
            Funds_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Funds_Label.loc = Game_Data_Window.loc + new Vector2(8, 24);
            Funds_Label.src_rect = new Rectangle(0, 128, 32, 16);
            Funds_Label.stereoscopic = Config.DATA_DATA_DEPTH;
            Gold_G = new Sprite();
            Gold_G.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Gold_G.loc = Game_Data_Window.loc + new Vector2(88 + 5, 24);
            Gold_G.src_rect = new Rectangle(0, 160, 16, 16);
            Gold_G.stereoscopic = Config.DATA_DATA_DEPTH;

            Lv_Label = new Sprite();
            Lv_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Lv_Label.loc = UnitBg.loc + new Vector2(8, 40 + 0 - 2);
            Lv_Label.src_rect = new Rectangle(32, 112, 16, 16);
            Lv_Label.stereoscopic = Config.DATA_LEADER_DEPTH;
            Hp_Label = new Sprite();
            Hp_Label.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            Hp_Label.loc = UnitBg.loc + new Vector2(8, 40 + 12 - 2);
            Hp_Label.src_rect = new Rectangle(48, 112, 24, 16);
            Hp_Label.stereoscopic = Config.DATA_LEADER_DEPTH;
            TextSprite hp_slash = new TextSprite(); // why is this done so //@Yeti
            hp_slash.loc = UnitBg.loc + new Vector2(44, 40 + 12);
            hp_slash.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            hp_slash.text = "/";
            Hp_Slash = hp_slash;
            Hp_Slash.stereoscopic = Config.DATA_LEADER_DEPTH;
            RatingLabel = new Sprite();
            RatingLabel.texture = Global.Content.Load<Texture2D>(@"Graphics/Windowskins/Data_Screen");
            RatingLabel.loc = UnitBg.loc + new Vector2(72, 40 + 12 - 2);
            RatingLabel.src_rect = new Rectangle(72, 112, 24, 16);
            RatingLabel.stereoscopic = Config.DATA_LEADER_DEPTH;

            RankingLabel = new TextSprite();
            RankingLabel.loc = RankingWindow.loc + new Vector2(24, 8);
            RankingLabel.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            RankingLabel.text = "Progress";
            RankingLabel.stereoscopic = Config.DATA_TEAMS_DEPTH;
            TurnsRankLabel = new TextSprite();
            TurnsRankLabel.loc = RankingWindow.loc + new Vector2(8, 24);
            TurnsRankLabel.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            TurnsRankLabel.text = "Turns";
            TurnsRankLabel.stereoscopic = Config.DATA_TEAMS_DEPTH;
            CombatRankLabel = new TextSprite();
            CombatRankLabel.loc = RankingWindow.loc + new Vector2(8, 40);
            CombatRankLabel.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            CombatRankLabel.text = "Combat";
            CombatRankLabel.stereoscopic = Config.DATA_TEAMS_DEPTH;
            ExpRankLabel = new TextSprite();
            ExpRankLabel.loc = RankingWindow.loc + new Vector2(8, 56);
            ExpRankLabel.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            ExpRankLabel.text = "Exp";
            ExpRankLabel.stereoscopic = Config.DATA_TEAMS_DEPTH;

            FileLabel = new TextSprite();
            FileLabel.loc = FileBg.loc + new Vector2(8, 40);
            FileLabel.SetFont(Config.UI_FONT, Global.Content, "Yellow");
            FileLabel.text = "File";
            FileLabel.stereoscopic = Config.DATA_LEADER_DEPTH;

            // Victory Text
            Victory_Text = new TextSprite();
            Victory_Text.loc = Objective_Window.loc + new Vector2(68, 12);
            Victory_Text.SetFont(Config.UI_FONT, Global.Content, "White");
            Victory_Text.text = Global.game_system.Victory_Text;
            Victory_Text.stereoscopic = Config.DATA_WINDOW_DEPTH;
            // Loss Text
            Loss_Text = new TextSprite();
            Loss_Text.loc = Objective_Window.loc + new Vector2(68, 44);
            Loss_Text.SetFont(Config.UI_FONT, Global.Content, "White");
            Loss_Text.text = Global.game_system.Loss_Text;
            Loss_Text.stereoscopic = Config.DATA_WINDOW_DEPTH;

            // Turn Text
            Turn = new RightAdjustedText();
            Turn.loc = Game_Data_Window.loc + new Vector2(100, 8);
            Turn.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Turn.text = Global.game_system.chapter_turn.ToString();
            Turn.stereoscopic = Config.DATA_DATA_DEPTH;
            // Funds Text
            Funds = new RightAdjustedText();
            Funds.loc = Game_Data_Window.loc + new Vector2(92, 24);
            Funds.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Funds.text = Global.battalion.gold.ToString();
            Funds.stereoscopic = Config.DATA_DATA_DEPTH;

            // Leader Name
            Leader_Name = new TextSprite();
            Leader_Name.loc = UnitBg.loc + new Vector2(32 + 24, 12);
            Leader_Name.SetFont(Config.UI_FONT, Global.Content, "White");
            Leader_Name.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Level
            Lvl = new RightAdjustedText();
            Lvl.loc = UnitBg.loc + new Vector2(68, 40 + 0);
            Lvl.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Lvl.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Hp
            Hp = new RightAdjustedText();
            Hp.loc = UnitBg.loc + new Vector2(44, 40 + 12);
            Hp.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Hp.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Hp Max
            Hp_Max = new RightAdjustedText();
            Hp_Max.loc = UnitBg.loc + new Vector2(68, 40 + 12);
            Hp_Max.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Hp_Max.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Rating
            Rating = new RightAdjustedText();
            Rating.loc = UnitBg.loc + new Vector2(120, 40 + 12);
            Rating.SetFont(Config.UI_FONT, Global.Content, "Blue");
            Rating.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Map Sprite
            Map_Sprite = new Character_Sprite();
            Map_Sprite.facing_count = 3;
            Map_Sprite.frame_count = 3;
            Map_Sprite.loc = UnitBg.loc + new Vector2(20, 24);
            Map_Sprite.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Miniface
            Face = new Miniface();
            Face.loc = UnitBg.loc + new Vector2(104, 8);
            Face.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Weapon
            LeaderWeapon = new Item_Display();
            LeaderWeapon.loc = UnitBg.loc + new Vector2(8, 28);
            LeaderWeapon.stereoscopic = Config.DATA_LEADER_DEPTH;

            // File number
            FileNumber = new RightAdjustedText();
            FileNumber.loc = FileBg.loc + new Vector2(40, 40);
            FileNumber.SetFont(Config.UI_FONT, Global.Content, "Blue");
            FileNumber.text = Global.current_save_id.ToString();
            FileNumber.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Style
            StyleText = new TextSprite();
            StyleText.loc = FileBg.loc + new Vector2(8, 8);
            StyleText.SetFont(Config.UI_FONT, Global.Content, "Blue");
            StyleText.text = Global.game_system.Style.ToString();
            StyleText.stereoscopic = Config.DATA_LEADER_DEPTH;
            // Difficulty
            DifficultyText = new TextSprite();
            DifficultyText.loc = FileBg.loc + new Vector2(8, 24);
            DifficultyText.SetFont(Config.UI_FONT, Global.Content, "Blue");
            DifficultyText.text = Global.game_system.Difficulty_Mode.ToString();
            DifficultyText.stereoscopic = Config.DATA_LEADER_DEPTH;

            // Rank Letters
            TotalRankLetter = new TextSprite();
            TotalRankLetter.loc = RankingWindow.loc + new Vector2(88, 8);
            TotalRankLetter.SetFont(Config.UI_FONT + "L", Global.Content, "Blue", Config.UI_FONT);
            TotalRankLetter.stereoscopic = Config.DATA_TEAMS_DEPTH;
            TurnsRankLetter = new TextSprite();
            TurnsRankLetter.loc = RankingWindow.loc + new Vector2(104, 24);
            TurnsRankLetter.SetFont(Config.UI_FONT + "L", Global.Content, "Blue", Config.UI_FONT);
            TurnsRankLetter.stereoscopic = Config.DATA_TEAMS_DEPTH;
            CombatRankLetter = new TextSprite();
            CombatRankLetter.loc = RankingWindow.loc + new Vector2(104, 40);
            CombatRankLetter.SetFont(Config.UI_FONT + "L", Global.Content, "Blue", Config.UI_FONT);
            CombatRankLetter.stereoscopic = Config.DATA_TEAMS_DEPTH;
            ExpRankLetter = new TextSprite();
            ExpRankLetter.loc = RankingWindow.loc + new Vector2(104, 56);
            ExpRankLetter.SetFont(Config.UI_FONT + "L", Global.Content, "Blue", Config.UI_FONT);
            ExpRankLetter.stereoscopic = Config.DATA_TEAMS_DEPTH;
            // Rank Gauges
            TurnsBar = new Stat_Bar();
            TurnsBar.loc = RankingWindow.loc + new Vector2(40, 32);
            TurnsBar.offset = new Vector2(-2, 0);
            TurnsBar.bar_width = RANKING_BAR_LENGTH;
            TurnsBar.stereoscopic = Config.DATA_TEAMS_DEPTH;
            CombatBar = new Stat_Bar();
            CombatBar.loc = RankingWindow.loc + new Vector2(40, 48);
            CombatBar.offset = new Vector2(-2, 0);
            CombatBar.bar_width = RANKING_BAR_LENGTH;
            CombatBar.stereoscopic = Config.DATA_TEAMS_DEPTH;
            ExpBar = new Stat_Bar();
            ExpBar.loc = RankingWindow.loc + new Vector2(40, 64);
            ExpBar.offset = new Vector2(-2, 0);
            ExpBar.bar_width = RANKING_BAR_LENGTH;
            ExpBar.stereoscopic = Config.DATA_TEAMS_DEPTH;

            // Page Arrows
            Left_Page_Arrow = new Page_Arrow();
            Left_Page_Arrow.loc = new Vector2(4, 72);
            Left_Page_Arrow.stereoscopic = Config.DATA_BANNER_DEPTH;
            Left_Page_Arrow.ArrowClicked += Left_Page_Arrow_ArrowClicked;
            Right_Page_Arrow = new Page_Arrow();
            Right_Page_Arrow.loc = new Vector2(Config.WINDOW_WIDTH - 4, 72);
            Right_Page_Arrow.mirrored = true;
            Right_Page_Arrow.stereoscopic = Config.DATA_BANNER_DEPTH;
            Right_Page_Arrow.ArrowClicked += Right_Page_Arrow_ArrowClicked;

            CreateCancelButton();

            refresh_rank();
            set_images();
        }

        private void CreateCancelButton()
        {
            CancelButton = Button_Description.button(Inputs.B,
                new Vector2(Config.WINDOW_WIDTH - 56, 8));
            CancelButton.description = "Cancel";
            CancelButton.stereoscopic = Config.UNIT_ARROWS_DEPTH;
        }

        private void refresh_rank()
        {
            var ranking = Global.game_state.calculate_ranking_progress();
            if (ranking == null)
            {
                refresh_ranking_letter(TotalRankLetter, "");
                refresh_ranking_letter(TurnsRankLetter, "-");
                refresh_ranking_letter(CombatRankLetter, "-");
                refresh_ranking_letter(ExpRankLetter, "-");
            }
            else
            {
                refresh_ranking_letter(TotalRankLetter, ranking.rank);
                refresh_ranking_letter(TurnsRankLetter, ranking.turns_letter);
                refresh_ranking_letter(CombatRankLetter, ranking.combat_letter);
                refresh_ranking_letter(ExpRankLetter, ranking.exp_letter);

                refresh_ranking_bar(TurnsBar, ranking.turns);
                refresh_ranking_bar(CombatBar, ranking.combat);
                refresh_ranking_bar(ExpBar, ranking.exp);
            }
        }

        private void refresh_ranking_letter(TextSprite text, string letter)
        {
            text.text = letter;
            if (letter == TactileLibrary.Chapters.DataRanking.best_rank)
                text.SetColor(Global.Content, "Green");
            else
                text.SetColor(Global.Content, "Blue");
        }
        private void refresh_ranking_bar(Stat_Bar bar, int score)
        {
            int bar_length = score * RANKING_BAR_LENGTH / 150;
            // 100% yellow bar with green bonus
            if (bar_length > RANKING_BAR_LENGTH * 2 / 3)
            {
                bar.SetFillWidth(RANKING_BAR_LENGTH * 2 / 3);
                bar.bonus_width = bar_length - bar.fill_width;
            }
            // Short yellow bar with red penalty
            else
            {
                bar.SetFillWidth(bar_length);
                bar.bonus_width = bar.fill_width - RANKING_BAR_LENGTH * 2 / 3;
            }
        }

        private void set_images()
        {
            if (!TeamNodes.Any())
            {
                Cursor.loc = TEAM_WINDOW_LOC;
                Cursor.height = 8;
            }
            else
            {
                Cursor.loc = TeamNodes.ActiveNode.loc;
                Cursor.height = (int)TeamNodes.ActiveNode.Size.Y + 8;
            }
            int leader_id = -1;
            if (Groups.Count > 0)
            {
                int team = Groups[TeamNodes.ActiveNodeIndex].Key;
                leader_id = Global.game_map.team_leaders[team];
                // If there's no player leader, use the first battalion member
                if (leader_id == -1 && team == Constants.Team.PLAYER_TEAM &&
                        Global.battalion.actors.Any())
                {
                    int actorId = Global.battalion.actors[0];
                    leader_id = Global.game_map.get_unit_id_from_actor(actorId);
                }
            }
            if (leader_id == -1 || Global.game_map.unit_defeated(leader_id) ||
                Global.game_map.unit_escaped(leader_id))
            {
                Leader_Name.text = "";
                Lvl.text = "--";
                Hp.text = "--";
                Hp_Max.text = "--";
                Rating.text = "--";
                Map_Sprite.texture = null;
                Face.set_actor("");
                LeaderWeapon.set_image(null, null);
            }
            else
            {
                Game_Unit unit = Global.game_map.units[leader_id];
                Leader_Name.text = unit.actor.name;
                Leader_Name.offset.X = Leader_Name.text_width / 2;
                Lvl.text = unit.actor.level.ToString();
                Hp.text = unit.actor.hp.ToString();
                Hp_Max.text = unit.actor.maxhp.ToString();
                Rating.text = unit.rating().ToString();
                Map_Sprite.texture = Scene_Map.get_team_map_sprite(unit.team, unit.map_sprite_name);
                if (Map_Sprite.texture != null)
                    Map_Sprite.offset = new Vector2(
                        (Map_Sprite.texture.Width / Map_Sprite.frame_count) / 2,
                        (Map_Sprite.texture.Height / Map_Sprite.facing_count) - 8);
                Map_Sprite.mirrored = unit.has_flipped_map_sprite;
                Face.set_actor(unit.actor);
                Face.mirrored = false;
                LeaderWeapon.set_image(unit.actor, unit.actor.weapon);
            }
        }

        #region Update
        private void update_map_sprite()
        {
            int old_frame = Map_Sprite_Frame;
            Map_Sprite_Frame = Global.game_system.unit_anim_idle_frame;
            if (Map_Sprite_Frame != old_frame)
                Map_Sprite.frame = Map_Sprite_Frame;
        }

        protected override void black_screen_switch()
        {
            Visible = !Visible;
            if (!_Closing)
            {
            }
        }

        protected override void UpdateMenu(bool active)
        {
            update_map_sprite();
            Cursor.update();
            Counter.update();
            Left_Page_Arrow.update();
            Right_Page_Arrow.update();

            int index = TeamNodes.ActiveNodeIndex;
            TeamNodes.Update(active && this.ready_for_inputs && page == 0, PageLoc);
            if (index != TeamNodes.ActiveNodeIndex)
            {
                set_images();
            }

            // Cancel button
            CancelButton.Update(active && this.ready_for_inputs);

            base.UpdateMenu(active);

            if (ChangingPage)
            {

                (PageLoc.X < target_page_loc.X ? Right_Page_Arrow : Left_Page_Arrow).twirling_update();
                float distance = Math.Abs(PageLoc.X - target_page_loc.X) / 2;
                distance = MathHelper.Clamp(MathHelper.Lerp(0, distance, ChangingPageTime / 12f), 0, distance);
                distance = Math.Min(distance, 32);
                distance = Math.Max(distance, 1);
                distance = (float)Math.Pow(2, Math.Round(Math.Log(distance, 2)));
                PageLoc.X = (float)Additional_Math.double_closer(PageLoc.X, target_page_loc.X, (int)distance);

                ChangingPageTime++;
                if (PageLoc.X == target_page_loc.X)
                    ChangingPage = false;
            }

            Left_Page_Arrow.visible = ChangingPage || page > 0;
            Right_Page_Arrow.visible = ChangingPage || page < 1;
        }

        protected override void UpdateAncillary()
        {
            if (Input.ControlSchemeSwitched)
            {
                if (CancelButton != null)
                    CreateCancelButton();
            }
        }

        protected override void update_input(bool active)
        {
            bool input = active && this.ready_for_inputs;
            if (input)
            {
                Left_Page_Arrow.UpdateInput();
                Right_Page_Arrow.UpdateInput();

                /*
                if (Global.Input.repeated(Inputs.Down))
                {
                    if (Index < Groups.Count - 1 && page == 0)
                    {
                        Global.game_system.play_se(System_Sounds.Menu_Move1);
                        move_down();
                    }
                }
                else if (Global.Input.repeated(Inputs.Up))
                {
                    if (Index > 0 && page == 0)
                    {
                        Global.game_system.play_se(System_Sounds.Menu_Move1);
                        move_up();
                    }
                }
                else */
                if (Global.Input.triggered(Inputs.Left) ||
                    Global.Input.gesture_triggered(TouchGestures.SwipeRight)) //Debug
                {
                    page_left();
                }
                else if (Global.Input.triggered(Inputs.Right) ||
                    Global.Input.gesture_triggered(TouchGestures.SwipeLeft)) //Debug
                {
                    page_right();
                }
                else if (Global.Input.triggered(Inputs.B) ||
                    Global.Input.gesture_triggered(TouchGestures.LongPress) ||
                    CancelButton.consume_trigger(MouseButtons.Left) ||
                    CancelButton.consume_trigger(TouchGestures.Tap)) //Debug
                {
                    Global.game_system.play_se(System_Sounds.Cancel);
                    close();
                }
            }
        }

        private void Left_Page_Arrow_ArrowClicked(object sender, EventArgs e)
        {
            page_left();
        }
        private void Right_Page_Arrow_ArrowClicked(object sender, EventArgs e)
        {
            page_right();
        }

        protected override void close()
        {
            base.close();
        }
        #endregion

        #region Movement
        private void page_left()
        {
            if (page > 0)
            {
                Global.game_system.play_se(System_Sounds.Status_Page_Change);
                page--;
                ChangingPage = true;
                ChangingPageTime = 0;
            }
        }

        private void page_right()
        {
            if (page < 1)
            {
                Global.game_system.play_se(System_Sounds.Status_Page_Change);
                page++;
                ChangingPage = true;
                ChangingPageTime = 0;
            }
        }
        #endregion

        #region Draw
        protected override void draw_background(SpriteBatch sprite_batch)
        {
            if (Background != null)
            {
                Effect background_shader = Global.effect_shader();
                if (background_shader != null)
                {
                    background_shader.CurrentTechnique = background_shader.Techniques["Technique2"];
                    background_shader.Parameters["color_shift"].SetValue(new Color(128, 0, 0, 64).ToVector4());
                }
                sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, null, null, null, background_shader);
                Background.draw(sprite_batch);
                sprite_batch.End();
            }
        }

        protected override void draw_window(SpriteBatch sprite_batch)
        {
            Vector2 offset = PageLoc;
            draw_leader_data(sprite_batch, offset);
            draw_file_data(sprite_batch, offset);
            if (page == 0 || ChangingPage)
                draw_team_data(sprite_batch, offset);
            if (page == 1 || ChangingPage)
            {
                draw_gameplay_data(sprite_batch, offset);
                draw_ranking_data(sprite_batch, offset);
            }
            draw_objective_data(sprite_batch, offset);

            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            // Banner
            BannerBg.draw(sprite_batch);
            Banner.draw(sprite_batch);
            CancelButton.Draw(sprite_batch);

            // Page Arrows
            if (page == 0 || ChangingPage)
                Right_Page_Arrow.draw(sprite_batch);
            if (page == 1 || ChangingPage)
                Left_Page_Arrow.draw(sprite_batch);
            sprite_batch.End();
        }

        private void draw_leader_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            // Team leader
            // Background
            UnitBg.draw(sprite_batch, offset);
            // Labels
            Lv_Label.draw(sprite_batch, offset);
            Hp_Label.draw(sprite_batch, offset);
            Hp_Slash.draw(sprite_batch, offset);
            RatingLabel.draw(sprite_batch, offset);
            sprite_batch.End();
            // Data
            Face.draw(sprite_batch, offset);

            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            Leader_Name.draw(sprite_batch, offset);
            Lvl.draw(sprite_batch, offset);
            Hp.draw(sprite_batch, offset);
            Hp_Max.draw(sprite_batch, offset);
            Rating.draw(sprite_batch, offset);
            LeaderWeapon.draw(sprite_batch, offset);
            Map_Sprite.draw(sprite_batch, offset);
            sprite_batch.End();
        }

        private void draw_file_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            // Team leader
            // Background
            FileBg.draw(sprite_batch, offset);
            // Labels
            FileLabel.draw(sprite_batch, offset);
            // Data
            FileNumber.draw(sprite_batch, offset);
            DifficultyText.draw(sprite_batch, offset);
            StyleText.draw(sprite_batch, offset);
            sprite_batch.End();
        }

        private void draw_gameplay_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            // Turns/Funds panel
            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            Game_Data_Window.draw(sprite_batch, offset);
            Turn_Label.draw(sprite_batch, offset);
            Funds_Label.draw(sprite_batch, offset);
            Gold_G.draw(sprite_batch, offset);
            Turn.draw(sprite_batch, offset);
            Funds.draw(sprite_batch, offset);
            Counter.draw(sprite_batch, offset);
            sprite_batch.End();
        }

        private void draw_team_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            // Team panels
            Effect background_shader = Global.effect_shader();
            if (background_shader != null)
            {
                background_shader.CurrentTechnique = background_shader.Techniques["Technique2"];
                background_shader.Parameters["color_shift"].SetValue(new Color(255, 255, 255, 32).ToVector4());
            }

            TeamNodes.Draw(sprite_batch, offset);

            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            Cursor.draw(sprite_batch, offset);
            sprite_batch.End();
        }

        private void draw_ranking_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            // Windows
            RankingWindow.draw(sprite_batch, offset);
            // Labels
            RankingLabel.draw(sprite_batch, offset);
            TurnsRankLabel.draw(sprite_batch, offset);
            CombatRankLabel.draw(sprite_batch, offset);
            ExpRankLabel.draw(sprite_batch, offset);
            // Data
            TurnsBar.draw(sprite_batch, offset);
            CombatBar.draw(sprite_batch, offset);
            ExpBar.draw(sprite_batch, offset);
            TotalRankLetter.draw(sprite_batch, offset);
            TurnsRankLetter.draw(sprite_batch, offset);
            CombatRankLetter.draw(sprite_batch, offset);
            ExpRankLetter.draw(sprite_batch, offset);
            sprite_batch.End();
        }

        private void draw_objective_data(SpriteBatch sprite_batch, Vector2 offset)
        {
            sprite_batch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            // Windows
            Objective_Window.draw(sprite_batch, offset);
            // Labels
            Objective_Label.draw(sprite_batch, offset);
            Defeat_Label.draw(sprite_batch, offset);
            // Data
            Victory_Text.draw(sprite_batch, offset);
            Loss_Text.draw(sprite_batch, offset);
            sprite_batch.End();
        }
        #endregion
    }
}
